/*======================================================================*
 *  Module .............libbt
 *  File ...............btcan.c
 *  Author .............Brian Zenowich
 *  Creation Date ......24 Mar 2003
 *                                                                      *
 *  ******************************************************************  *
 *                                                                      *
 *  NOTES:
 *
 *  REVISION HISTORY:
 *  24 Mar 2003 - BZ
 *    File created & documented.      
 *                                                                      *
 *======================================================================*/
#ifndef _BTCAN_H
#define _BTCAN_H
#ifdef __cplusplus
extern "C"
{
#endif/* __cplusplus */
//#include <defs.h>
//#include <semaphore.h>
//#include <dpm.h>
#include <inttypes.h>
#include <pthread.h>

/* #define D(x) <blank> ...for no debug info */
/* #define D(x) x       ...for debug info */
#define D(x) 
#define WAM_GOD 	 (10)
#define SAFETY_MODULE (10)
#define MAX_NODES    (16)

/* Public Data Structures */
/** CAN device information data structure
*/
typedef struct CANdev_t{
    int	CANfd; /**< File descriptor for the CAN device */
    int	channel; /**< Channel of the CAN device (normally 0) */
    pthread_mutex_t mutex; /** Thread mutex to insure only one thread at a time can use the device*/
}CANdev_t;

/* (internal) This keeps data that is local to each instance of the ICS server */
typedef struct local_info_t {
    int         icfd;            /* handle for ICAN device */
    int         fd0;             /* handle for output device (read) */
    int         fd;              /* handle for output device (write)*/
    pthread_t   ican_task;       /* task id for message thread */
    pthread_t   fjob_task;       /* test job : task id */
    int         fjob_arg[16];    /* test job : arguments */
    int         interpret;       /* Should use 2controller handling */
    uint64_t	cps;			 /* Cycles Per Second on this machine */
} local_info_t ;

/* Public Functions */
int initCAN(char *device, int *fd);
int wakePuck(CANdev_t *CANdev, int who);
int setPacked(CANdev_t *CANdev, int group, int property, int *values);
int setProperty(CANdev_t *CANdev, int who, int property, long value);
int getProperty(CANdev_t *CANdev, int who, int property, long *reply);
int getGroupPropertyFast(CANdev_t *CANdev, int group, int num2wait4, int property, long *reply);
int downloadFirmware(CANdev_t *CANdev, int who, char *fname);
int getBusStatus(CANdev_t *CANdev, int *status);
int setProperty_noVerify(CANdev_t *CANdev, int who, int property, long value);

#define S08 (1 << 0)
#define U08 (1 << 1)
#define S16 (1 << 2)
#define U16 (1 << 3)
#define S24 (1 << 4)
#define U24 (1 << 5)
#define S32 (1 << 6)
#define U32 (1 << 7)
#define VIS (1 << 8)
#define SAV (1 << 9)
#define VISSAV (3 << 8)

/*! bcastGroup */
enum {
    WHOLE_ARM = 0,
    LOWER_ARM = -1,
    UPPER_ARM = -2
};

/*! Properties for GOD and Taters */
enum {
    FIRMWARE_VERSION,
    ERR,             /* General Software Error */
    STATUS,          /* Software status */
    CONTROL_MODE,    /* Unused in WAMGOD */
    MEMORY_ADDRESS,  /* Address to poke/peek */
    MEMORY_VALUE     /* Value to poke, value peeked */
};

/*! Properties for the safety module */
enum {
    BUS_STATE = 6,       /* -1 = BUS_ERROR, 0 = BUS_OFF, 1 = BUS_ON */
    BUS_ERR,         /* [RRRRRRRVVVVVVVEH], Reset, Velocity, E-Stop, Heartbeat */
    SAVED_BUS_ERR,   /* First BUS_ERR to trigger a fault */
    RELAY_STATE,     /* 0 = Off, 1 = Ready (waiting for enable), 3 = Running */
    HEARTBEAT_MS,    /* Milliseconds before heartbeat time registers as failure */
    CONTROLLERS,     /* Listen for heartbeats from the host and controllers in this bitfield */
    MAX_ENABLE_VELOCITY,   /* Max velocity of motors allowed before EnableBus() */
    MAX_ENABLE_TORQUE,     /* Max commanded torque allowed before EnableBus() */
    MAX_VEL_1,             /* Max allowed velocity of motor n */
    MAX_VEL_2,
    MAX_VEL_3,
    MAX_VEL_4,
    MAX_VEL_5,
    MAX_VEL_6,
    MAX_VEL_7,
    VEL_1,                  /* Present velocity of motor n (float) */
    VEL_2,
    VEL_3,
    VEL_4,
    VEL_5,
    VEL_6,
    VEL_7
};

enum bus_status_enum{
    BUS_ERROR = -1,
    BUS_OFF,
    BUS_ON
};
  
/*! Properties for Taters */
enum tater_property_enum{
    COMMANDED_TORQUE = 6,   /*!< Q4.12 torque commanded */
    MAX_TORQUE,         /*!< Q4.12 maximum torque allowed */
    ACTUAL_POSITION,    /*!< Position in encoder cts */
    MECH_ANGLE,         /*!< Position relative to encoder offset of encoder index from electrical cycle start */
    DRIVE_MODE,        /*!< 0 = No hall change or encoder pulse found yet. 1 = Hall change found but no encoder pulse yet. 2 = Encoder pulse found, fully operational.*/
    IS_ZEROED,          /*!< Defaults to 0, is set to 1 when a "set position x" is received */
    MOTOR_OFFSET,       /*!< Encoder counts between encoder index and electrical cycle start */
    MOTOR_TEMPERATURE,  /*!< Present temperature of the motor */
    MAX_TEMPERATURE,    /*!< Max allowed temperature before error is generated */
    A_TO_D,             /*!< Present A/D reading */
    I_OFFSET_B,         /*!< The Current Offset for the B-Phase */
    I_OFFSET_C,         /*!< The Current Offset for the C-Phase */
    GROUP_A,            /*!< The first group to which this puck belongs (usually 0, broadcast) */
    GROUP_B,            /*!< The second group to which this puck belongs */
    GROUP_C,            /*!< The third group to which this puck belongs */
    PACK_IDX,           /*!< This puck's index into the packed values of received data */
    CTS_PER_REV,        /*!< Encoder counts per revolution */
    POSITION_OFFSET     /*!< Difference between the arbitrary zero point and the encoder's index pulse */
  
};

enum {
    STATUS_OFFLINE = -1,
	STATUS_RESET,
	STATUS_SHUTDOWN,
	STATUS_INITIALIZE,
	STATUS_ERR,
	STATUS_READY,
	STATUS_COMMUTATING
};

enum {
	ERR_NONE,
	ERR_READ_ONLY,
	ERR_OUT_OF_RANGE
};
/*! Control_mode states */
enum {
	MODE_IDLE,
	MODE_DUTY_CYCLE,
	MODE_TORQUE,
	MODE_PID,
	MODE_VELOCITY,
	MODE_TRAPEZOIDAL
};

#ifdef __cplusplus
}
#endif/* __cplusplus */
#endif _BTCAN_H

/*======================================================================*
 *                                                                      *
 *             Copyright (c) 2003 Barrett Technology, Inc.              *
 *                        139 Main Street                               *
 *                       Kendall/MIT Square                             *
 *                  Cambridge, MA  02142-1528  USA                      *
 *                                                                      *
 *                        All rights reserved.                          *
 *                                                                      *
 *  ******************************************************************  *
 *                            DISCLAIMER                                *
 *                                                                      *
 *  This software and related documentation are provided to you on      *
 *  an as is basis and without warranty of any kind.  No warranties,    *
 *  express or implied, including, without limitation, any warranties   *
 *  of merchantability or fitness for a particular purpose are being    *
 *  provided by Barrett Technology, Inc.  In no event shall Barrett     *
 *  Technology, Inc. be liable for any lost development expenses, lost  *
 *  lost profits, or any incidental, special, or consequential damage.  *
 *  ******************************************************************  *
 *                                                                      
 * CVS info: $Id$
 * CVS automatic log (prune as desired):
 * $Log$
 *
 *======================================================================*/