/*
   Module .............libbt                                
   File ...............gimbals.c                                          
   Author .............Brian Zenowich                                 
   Creation Date ......Oct 31, 2003                                 
                                                                       
              Copyright (c) 2003 Barrett Technology, Inc.              
                         625 Mt. Auburn St.                             
                   Cambridge, MA  02138-4555  USA                      
                                                                      
                         All rights reserved.                          
                                                                       
 ***********************************************************************
 *                            DISCLAIMER                               *
 *                                                                     *
 *  This software and related documentation are provided to you on     *
 *  an as is basis and without warranty of any kind.  No warranties,   *
 *  express or implied, including, without limitation, any warranties  *
 *  of merchantability or fitness for a particular purpose are being   *
 *  provided by Barrett Technology, Inc.  In no event shall Barrett    *
 *  Technology, Inc. be liable for any lost development expenses, lost *
 *  lost profits, or any incidental, special, or consequential damage. *
 *                                                                     *
 ***********************************************************************  
 *                                                                      
 *  NOTES:                                                              
 *   Reads the gimbles angles, initializes the gimbals if necessary                        
 *                                                                      
 *  REVISION HISTORY:                                                   
 *  031031 - BZ - File created                                          
 *                                                                      
 *======================================================================*/

/* \file gimbals.c  
   \brief Reads the gimbles angles, initializes the gimbals if necessary
    
    
*/ 

#include "btcan.h"
#include "btsystem.h"
#include "stdio.h"
#include "syslog.h"

int gimbalsInit = 0;
double gimbalsOffset[3];
double gimbalsGain[3];

int getGimbalsAngles(double *gimbals)
{
    long int temp[8]; /* A temp array to store the gimbals positions */
    FILE *inFile;
    bus_struct *bus;
    actuator_struct *act;
    int num_bus, num_act;
    int i;
    
    act = GetActuators(&num_act); //get pointer and number of actuators
    bus = GetBuses(&num_bus); //get pointer and number of buses.
    
    if(!gimbalsInit)
    {
        /* Wake the gimbals puck */
        wakePuck(&(bus[act[0].bus].CANdev), 5);
        
        /* Read the gain/offset information from the gimbals.dat file */
        if((inFile = fopen("gimbals.dat", "r")) == NULL)
        {
            syslog(LOG_ERR, "Could not open gimbals.dat file!");
            return(1);
        }
        for(i = 0; i < 3; i++)
        {
            fscanf(inFile, "%lf %lf", &gimbalsOffset[i], &gimbalsGain[i]);
        }
        fclose(inFile);
        
        gimbalsInit = 1;
    }
    
    /* Read the gimbles puck for its three positions (A/D readings) */
    getGroupPropertyFast(
        &(bus[act[0].bus].CANdev)   /** Pointer to the CANdev_t structure */, 
        5                   /** Group number of pucks whose property we want */, 
        3                           /** Number of pucks in this group (number of responses to wait for) */, 
        ACTUAL_POSITION             /** The property to get */, 
        temp                     /** An array of replies indexed by the Node ID */);

    /* Extract the gimbals position information from the temp array */
    gimbals[0] = (temp[5] - gimbalsOffset[0]) / gimbalsGain[0];
    gimbals[1] = (temp[6] - gimbalsOffset[1]) / gimbalsGain[1];
    gimbals[2] = (temp[7] - gimbalsOffset[2]) / gimbalsGain[2];
    
    return(0); /* Return success */
}

/*======================================================================*
 *                                                                      *
 *             Copyright (c) 2003 Barrett Technology, Inc.              *
 *                        139 Main Street                               *
 *                       Kendall/MIT Square                             *
 *                  Cambridge, MA  02142-1528  USA                      *
 *                                                                      *
 *                        All rights reserved.                          *
 *                                                                      *
 *  ******************************************************************  *
 *                            DISCLAIMER                                *
 *                                                                      *
 *  This software and related documentation are provided to you on      *
 *  an as is basis and without warranty of any kind.  No warranties,    *
 *  express or implied, including, without limitation, any warranties   *
 *  of merchantability or fitness for a particular purpose are being    *
 *  provided by Barrett Technology, Inc.  In no event shall Barrett     *
 *  Technology, Inc. be liable for any lost development expenses, lost  *
 *  lost profits, or any incidental, special, or consequential damage.  *
 *  ******************************************************************  *
 *                                                                      
 * CVS info: $Id$
 * CVS automatic log (prune as desired):
 * $Log$
 *
 *======================================================================*/















